<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use App\Exception;

class OperatorController {

  private $operatorService;
  private $jwt;
  private $util;
  
  public function __construct (\App\Service\OperatorService $operatorService,
                                \App\Middlewares\JwtManager $jwt,
                                \App\Util\Util $util
                               ) {

      $this->operatorService = $operatorService;
      $this->jwt  = $jwt;
      $this->util = $util;
    }

    public function operatorLoginController(Request $request, Response $response) {
      try {
          $params = $request->getParsedBody();

          if(empty($params['email'])) {
              throw new \Exception('E-mail do operador não informado');
          }

          if(empty($params['password'])) {
              throw new \Exception('Senha do operador não informado');
          }

          $email       = $params['email'];
          $password    = $params['password'];
          //$password    = $this->util->encrypt($params['password']);
          
          $dataOperator = $this->operatorService->operatorLoginService($email, $password);

          if(empty($dataOperator)) {
            throw new \Exception('Usuário/Senha inválido.');
          }

          $token = $this->jwt->jwtOperatorLogin($dataOperator);

          $response->getBody()->write(json_encode([
            'error'   => false,
            'message' => '',
            'data'    => [
              'token' => $token,
              'operator' => $dataOperator
            ]
        ]));
        return $response->withHeader('Content-Type', 'application/json');
      }catch(\Exception $e) {
        $response->getBody()->write(json_encode([
          'error'   => true,
          'message' => $e->getMessage(),
          'data'    => []
        ]));
        return $response;
      }
    }

    public function updatePassword(Request $request, Response $response) {
      try {
        $params          = $request->getParsedBody();
        $currentPassword = $params['currentPassword'];
        $newPassword	   = $params['newPassword'];
        $operatorEmail	 = $params['email'];
        $operatorName    = $params['operatorName'];
        $operatorId      = $params['operatorId'];

        $resultUpdatePassword = $this->operatorService->updatePassword($currentPassword, $newPassword, $operatorEmail, $operatorName, $operatorId);

        if($resultUpdatePassword['error']) {
          throw new \Exception($resultUpdatePassword['message']);
        }

        if($resultUpdatePassword['sendEmail'] !== null && $resultUpdatePassword['sendEmail']['error']) {
          $message = $resultUpdatePassword['message'] . '</br>' . $resultUpdatePassword['sendEmail']['message'];
        }

        $response->getBody()->write(json_encode([
          'error'     => $resultUpdatePassword['error'],
          'message'   => empty($message) ? $resultUpdatePassword['message'] : $message,
          'data'      => '',
          'sendEmail' => $resultUpdatePassword['sendEmail']
        ]));

        return $response->withHeader('Content-Type', 'application/json');
    } catch (\Exception $e) {
        $response->getBody()->write(json_encode([
            'error'   => true,
            'message' => $e->getMessage(),
            'data'    => []
        ]));

        return $response->withHeader('Content-Type', 'application/json');
    }
  }

    public function sendResetPasswordEmail(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();

            $email = !empty($params['email']) ? $params['email'] : null;

            if(empty($email)) {
                throw new \Exception("Email não foi informado.");
            }

            $dataOperator = $this->operatorService->validateEmail($email);
            $operatorName = $dataOperator['0']['NAME'];
            $token = $this->jwt->jwtResetPassword($dataOperator);
            $this->operatorService->resetPasswordEmail($email["form-email"], $operatorName, $token);
            

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => 'Email enviado com sucesso. Cheque a sua caixa de mensagens.',
                'data' => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function resetPassword(Request $request, Response $response) {
      try {
        $params = $request->getParsedBody();
        $token = $params[1];
        $dataOperator = $this->jwt->getJwtTokenData($token);
        if (!$dataOperator) {
          throw new \Exception('Token invalido');
        }

        $password = $params[0]['password'];
        $passwordConfirm = $params[0]['passwordConfirm'];

        if (empty($password) || empty($passwordConfirm)) {
          throw new \Exception('Senha não informada');
        }

        if ($password !== $passwordConfirm) {
          throw new \Exception('As senhas são diferentes');
        }
        $operatorName = $dataOperator->operator->nome;
        $operatorId = $dataOperator->operator->id;
        $operatorEmail = $dataOperator->operator->email;
        $this->operatorService->resetPassword($password, $operatorId, $operatorEmail, $operatorName);

        $response->getBody()->write(json_encode([
          'error'   => false,
          'message' => 'Senha alterada com sucesso.',
          'data'    => []
        ]));
        return $response->withHeader('Content-Type', 'application/json');
      }catch(\Exception $e) {
        $response->getBody()->write(json_encode([
          'error'   => true,
          'message' => $e->getMessage(),
          'data'    => []
        ]));
        return $response;
      }
    }
  }