<?php
namespace App\Service;

use App\Exception;
class OperatorService {

    private $util;
    private $operatorFactory;
	private $smtp;

    public function __construct (\App\Util\Util $util,
                                \App\Factory\OperatorFactory $operatorFactory,
                                \App\Service\SMTPService $smtp
                                ) {

        $this->util             = $util;
        $this->operatorFactory  = $operatorFactory;
        $this->smtp             = $smtp;
    }

    public function operatorLoginService($email, $password) {
        $endpoint = $this->util->getEnv("LOGIN_ENDPOINT");
        $requestData = [
            "requestType" => "FilterData",
            "filter" => [
                [
                    "name" => "LOGIN",
                    "operator" => "=",
                    "value" => $email
                ],
                [
                    "name" => "PASSWORD",
                    "operator" => "=",
                    "value" => $password
                ]
            ]
        ];

        $options = [
            'http' => [
                'method'  => 'POST',
                'header'  => 'Content-Type: application/json',
                'content' => json_encode($requestData),
            ],
        ];

        $context  = stream_context_create($options);
        $response = file_get_contents($endpoint, false, $context);
        $dataOperator = json_decode($response, true);

        // $dataOperator = $this->operatorFactory->login($email, $password);

        return $dataOperator['dataset']['operador'];
    }

    public function updatePassword($currentPassword, $newPassword, $operatorEmail, $operatorName, $operatorId) { 
        $currentPassword = $this->util->encrypt($currentPassword);
        $toCheckPassword = $this->operatorFactory->getPassword($operatorId);

        if ($currentPassword !== $toCheckPassword[0]['CDSENHAOPERWEB']) {
            return array(
                'error' => true,
				'message' => 'Senha atual inválida.'
			);
		}

        $encryptedNewPassword       = $this->util->encrypt($newPassword);
        $encryptedNewPasswordDelphi = $this->util->getDelphiPasswordEncrypted($newPassword, true);

        if($encryptedNewPassword === $toCheckPassword[0]['CDSENHAOPERWEB']) {
            return array(
                'error' => true,
				'message' => 'Digite uma senha diferente da atual.'
			);
        }

        $updatePassword = $this->operatorFactory->updatePassword($encryptedNewPassword, 
             $encryptedNewPasswordDelphi, $operatorId);

        $rememberEmail = $this->rememberPasswordEmail($operatorEmail, $operatorName);

        if ($updatePassword->rowCount() > 0) {
            return array(
			    'error'   => false,
			    'message' => 'Sua senha foi alterada com successo.',
                'sendEmail'   => $rememberEmail
		    );
        } else {
           return array(
                'error'   => true,
                'message' => 'Desculpe, ocorreu um erro ao tentar atualizar a senha. Por favor, verifique os dados inseridos e tente novamente.',
                'sendEmail'   => $rememberEmail
           ); 
        }
    }

    public function rememberPasswordEmail($operatorEmail, $operatorName) {
        $subject = 'Alteração de Senha';
        $content = file_get_contents(__DIR__ . "/../../templates/rememberPasswordEmailTemplate.html");
        return $this->smtp->sendEmail($operatorName, $operatorEmail, $subject, $content);
    }

    public function resetPasswordEmail($DSEMAILOPER, $operatorName, $token) {
        $subject = 'Redefinição de Senha';
        $content = file_get_contents(__DIR__ . "/../../templates/resetPasswordEmailTemplate.html");
        $content = str_replace(['{token}', '{controlURL}'], [$token, $this->util->getenv("CONTROL_URL")], $content);
        return $this->smtp->sendEmail($operatorName, $DSEMAILOPER, $subject, $content);
    }

    public function validateEmail ($email) {
        $dataOperator = $this->operatorFactory->validateEmail($email);

        if(empty($dataOperator)) {
            throw new \Exception("Email não encontrado.");
        }

        return $dataOperator;
    }

    public function resetPassword ($password, $id, $DSEMAILOPER, $operatorName) {
        $delphiPassword = $this->util->getDelphiPasswordEncrypted($password);
        $encryptedPassword = $this->util->encrypt($password);
        $updatePassword = $this->operatorFactory->updatePassword($encryptedPassword, $delphiPassword, $id);

        $rememberEmail = $this->rememberPasswordEmail($DSEMAILOPER, $operatorName);

        if ($updatePassword->rowCount() > 0) {
            return array(
			    'error'   => false,
			    'message' => 'Sua senha foi alterada com successo.',
                'sendEmail'   => $rememberEmail
		    );
        }
    }
}
